* Program:  HP7475A.PRG
* Author.:  John Kaster
* Date...:  7/21/1987
* Version:  1.10
* Notes..:  These utilities use an assembled OBJect file to address the
*           COM port.
*           These utilities are for the Hewlett-Packard HP7475A plotter
*           specifically and any commands you can use with any other
*           plotter is purely lucky coincidence, although these are generally
*           HPGL commands.
*
*           Functions contained in this library
*           Name               Description
*           =========          =========================================
*           HAAltChr           Designate alternate character set
*           HAArcAbs           Arc Absolute
*           HAArcRel           Arc Relative
*           HACharRel          Set character width & height
*           HACharSize         Size of characters
*           HACharSlant        Slant characters at an angle
*           HACircle           Draw a circle
*           HAClrDig           Clear Digitizing
*           HACmnd             Parses a command for the plotter
*           HADefault          Set plotter to default conditions
*           HADigPoint         Set plotter for digitize mode
*           HAEdgeAbs          Edge of rectangle in absolute coordinates
*           HAEdgeRel          Edge of rectangle in relative coordinates
*           HAEdgeWedge        Draws the edge of a wedge
*           HAFillRecA         Shade a rectangle using absolute coords
*           HAFillRecR         Shade a rectangle using relative coords
*           HAFillType         Type of area fill for RA, RR, or WG
*           HAFillWed          Shade a wedge
*           HAInitialize       Initialize the plotter
*           HAInputPs          Sets scaling points
*           HAInputWin         Sets plotting window
*           HALabel            Plots a label
*           HALabelAbs         Set direction for labels absolute
*           HALabelRel         Set direction for labels relative
*           HALineType         Defines line in drawing points
*           HAMovByChar        Move by character
*           HAPaperSize        Set A or B size paper
*           HAPenDown          Put the pen down
*           HAPenThick         Set pen thickness for filling in
*           HAPenUp            Put the pen up
*           HAPlotAbs          Plot in absolute coordinates
*           HAPlotRel          Plot in relative coordinates
*           HAPosAbs           Position pen in absolute coords
*           HAPosRel           Position pen in relative coords
*           HARotateCoor       Rotates the coordinate system ( 0 or 90)
*           HAScaling          Set scaling (user defined units)
*           HASelPen           Select a pen
*           HASelStand         Select standard character set
*           HAStandChr         Designate standard character set
*           HASymbMode         Specified symbol drawn at each plotted point
*           HATickLen          Set tick length for XT and YT
*           HAUserChar         Draw a user-defined character - not done
*           HAVelocity         Set pen velocity
*           HAXTick            Tick along X axis as defined by tick length
*           HAYTick            Tick along Y axis as defined by tick length
*           NTRIM              Returns ALLTRIM(EXPN)
*

FUNCTION NTRIM
* Syntax.:  NTrim( <ExpN> )
* Returns:  Alltrim( <ExpN> ) or Alltrim( <ExpC> )
PARA N
IF PCOUNT()<1
  RETURN ( "" )
ENDIF
IF TYPE('N')='N'
  RETURN ( LTRIM(TRIM(STR(N))) )
ENDIF
RETURN ( LTRIM(TRIM(N)) )

FUNCTION HACmnd
* Syntax:  HACmnd (<Port>,<HPGL Text> [, <List of numbers> ] )
* Notes.:  Sends text to the specified com port
PARA PlotPort,t,n1,n2,n3,n4,n5,n6,n7,n8,n9
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
PRIV I,ic,last
Last=PCount()
PlotPort=HACommPort(PlotPort)
t=upper(t)
IF Last>2
  t=t+ntrim(n1)
ENDIF
FOR I=4 TO Last
  ic='n'+ntrim(i-2)
  t=t+","+ntrim(&ic)
NEXT
CALL SET_XOFF WITH .T.
DO WHIL Get_XOff()
ENDDO
CALL OUTP_CHAR WITH t+";"
? T+';'
RETURN ( .T. )

FUNCTION HAArcAbs
* Draws arc of specified number of degrees with specified smoothness; centered
* at X,Y coordinate, using current pen status (up or down)
*
PARA PlotPort,x,y,arc,chord
IF PCOUNT()<4
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'AA',x,y,arc,chord)
RETURN ( .T. )

FUNCTION HAArcRel
* Draws arc of specified number of degrees with specified smoothness; centered
* relative to current pen position, using current pen status (up or down)
*
PARA PlotPort,x,y,arc,chord
IF PCOUNT()<4
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'AR',x,y,arc,chord)
RETURN ( .T. )

FUNCTION HACharSize
PARA PlotPort,width,height
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'SI',width,height)
RETURN ( .T. )

FUNCTION HAMovByChr
PARA PlotPort,spaces,lines
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'CP',spaces,lines)
RETURN ( .T. )

FUNCTION HAClrDig
* Clears digitize mode
HACmnd (PlotPort,'DC')
RETURN ( .T. )

FUNCTION HAEdgeAbs
* Edge rectangle absolute
PARA PlotPort,x,y
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'EA',x,y)
RETURN ( .T. )

FUNCTION HAEdgeRel
* Edge rectangle relative
PARA PlotPort,x,y
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'ER',x,y)
RETURN ( .T. )

FUNCTION HADefault
* Returns plotter to default conditions
PARA PlotPort
IF PCOUNT()<1
  PlotPort=1
ENDIF
HACmnd (PlotPort,'DF')
RETURN ( .T. )

FUNCTION HALabelAbs
* Sets the direction for labels absolute
PARA PlotPort,run,rise
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'DI',run,rise)
RETURN ( .T. )

FUNCTION HADigPoint
* Digitize a point
PARA PlotPort
HACmnd (PlotPort,'DP')
RETURN ( .T. )

FUNCTION HALabelRel
PARA PlotPort,run,rise
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'DR',run,rise)
RETURN ( .T. )

Function HACommPort
PARA PlotPort
IF TYPE('PlotPort')='N'
  RETURN ( 'COM'+NTRIM(PlotPort) )
ENDIF
RETURN ( PlotPort )

FUNCTION HAPaperSize
PARA PlotPort,PaperType
IF PCOUNT()<1
  RETURN ( .F. )
ENDIF
PaperType=UPPER(PaperType)
PlotPort=HACommPort(PlotPort)
DO CASE
  CASE PaperType = 'A'
    HACmnd (PlotPort,'PS4;IW0,0,10365,7962')
  CASE PaperType = 'B'
    HACmnd (PlotPort,'PS0;IW0,0,16640,10365')
  ENDC
RETURN ( .T. )

FUNCTION HAPenThick
PARA PlotPort,s
IF PCOUNT()<1
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'PT',s)
RETURN ( .T. )

FUNCTION HAScaling
PARA PlotPort,x1,x2,y1,y2
IF PCOUNT()<4
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'SC',x1,x2,y1,y2)
RETURN ( .T. )

FUNCTION HAStandChr
PARA PlotPort,I
IF PCOUNT()<1
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'CS,',i)
RETURN ( .T. )

FUNCTION HAAltChar
PARA PlotPort,I
IF PCOUNT()<1
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'CA',i)
RETURN ( .T. )

FUNCTION HAEdgeWedge
PARA PlotPort,radius,startA,sweepA,chord
IF PCOUNT()<3
  RETURN ( .F. )
ENDIF
IF PCOUNT()<4
  HACmnd (PlotPort,'EW',radius,StartA,SweepA)
ELSE
  HACmnd (PlotPort,'EW',radius,StartA,SweepA,chord)
ENDIF
RETURN ( .T. )

FUNCTION HAFillType
PARA PlotPort,FillType,Spacing,Angle
IF PCOUNT()<1
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'FT',FillType,spacing,angle)
RETURN ( .T. )

FUNCTION HAInitialize
PARA PlotPort
IF PCOUNT()<1
  PlotPort=1
ENDIF
HACmnd (PlotPort,'IN')
RETURN ( .T. )

FUNCTION HAInputPs
PARA PlotPort,P1x,P1y,P2x,P2y
IF Pcount()<1
  PlotPort=1
ENDIF
DO CASE
CASE PCOUNT()<2
  HACmnd(PlotPort,'IP')
CASE PCOUNT()>3
  HACmnd (PlotPort,'IP',P1x,P1y,P2x,P2y)
OTHE
  HACmnd (PlotPort,'IP',P1x,P1y)
ENDC
RETURN ( .T. )

FUNCTION HAInputWin
PARA PlotPort,X1,Y1,X2,Y2
DO CASE
CASE PCOUNT()<2
  HACmnd (PlotPort,'IW')
CASE PCOUNT()>3
  HACmnd (PlotPort,'IW',X1,Y1,X2,Y2)
OTHER
  HACmnd (PlotPort,'IW',X1,Y1)
ENDCASE
RETURN ( .T. )

FUNCTION HALabel
PARA PlotPort,Words
IF PCOUNT()<1
  PlotPort=1
ENDIF
HACmnd (PlotPort,'LB',Words+chr(3))
RETURN ( .T. )

FUNCTION HALineType
PARA PlotPort,I,Length
DO CASE
CASE PCOUNT()=0
  HACmnd (1,'LT')
CASE PCOUNT()=1
  HACmnd (PlotPort,'LT')
CASE PCOUNT()=2
  HACmnd (PlotPort,'LT',i)
CASE PCOUNT()=3
  HACmnd (PlotPort,'LT',i,Length)
ENDC
RETURN ( .T. )

FUNCTION HAPlotAbs
PARA PlotPort,X1,Y1,X2,Y2
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
IF PCOUNT()<5
  HACmnd(PlotPort,'PD')
  HACmnd(PlotPort,'PA',X1,Y1)
  HACmnd(PlotPort,'PU')
ELSE
  HACmnd(PlotPort,'PA;PU',X1,Y1)
  HACmnd(PlotPort,'PD')
  HACmnd(PlotPort,'PA',X1,Y1,X2,Y2)
  HACmnd(PlotPort,'PU')
ENDIF
RETURN ( .T. )

FUNCTION HAPenDown
PARA PlotPort,X,Y
IF PCOUNT()<1
  PlotPort=1
ENDIF
IF PCOUNT()<3
  HACmnd (PlotPort,'PD')
ELSE
  HACmnd (PlotPort,'PD',X,Y)
ENDIF
RETURN ( .T. )

FUNCTION HAPlotRel
PARA PlotPort,X1,Y1,X2,Y2
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
IF PCOUNT()<5
  HACmnd(PlotPort,'PD')
  HACmnd(PlotPort,'PR',X1,Y1)
  HACmnd(PlotPort,'PU')
ELSE
  HACmnd(PlotPort,'PR;PU',X1,Y1)
  HACmnd(PlotPort,'PD')
  HACmnd(PlotPort,'PR',X1,Y1,X2,Y2)
  HACmnd(PlotPort,'PU')
ENDIF
RETURN ( .T. )

FUNCTION HAPenUp
PARA PlotPort,X,Y
IF PCOUNT()<1
  PlotPort=1
ENDIF
IF PCOUNT()<3
  HACmnd (PlotPort,'PU')
ELSE
  HACmnd (PlotPort,'PU',X,Y)
ENDIF
RETURN ( .T. )

FUNCTION HAFillRecA
PARA PlotPort,X1,Y1,X2,Y2
IF PCOUNT()<5
  RETURN ( .F. )
ENDIF
HACmnd(PlotPort,'RA',X1,Y1,X2,Y2)
RETURN ( .T. )

FUNCTION HARotateCoor
PARA PlotPort,X
DO CASE
CASE PCOUNT()<1
  HACmnd(1,'RO')
CASE PCOUNT()<2
  HACmnd(PlotPort,'RO')
OTHE
  HACmnd(PlotPort,'RO',X)
ENDC
RETURN ( .T. )

FUNCTION HAFillRecR
PARA PlotPort,X1,Y1,X2,Y2
IF PCOUNT()<5
  RETURN ( .F. )
ENDIF
HACmnd(PlotPort,'RR',X1,Y1,X2,Y2)
RETURN ( .T. )

FUNCTION HACharSlant
PARA PlotPort,Tangent
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd(PlotPort,'SL',Tangent)
RETURN ( .T. )

FUNCTION HASymbMode
PARA PlotPort,CharVal
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd(PlotPort,'SM',CharVal)
RETURN ( .T. )

Function HATickLen
PARA PlotPort,Tick1,Tick2
PRIV I
I=PCOUNT()
IF I<1
  PlotPort=1
ENDIF
IF I<2
  Tick1=0.5
ENDIF
IF I<3
  Tick2=Tick1
ENDIF
HACmnd(PlotPort,'TL',Tick1,Tick2)
RETURN ( .T. )

FUNCTION HASelPen
PARA PlotPort,Pen
DO CASE
CASE PCOUNT()<1
  HACmnd(1,'SP')
CASE PCOUNT()<2
  HACmnd(PlotPort,'SP')
OTHE
  HACmnd(PlotPort,'SP',Pen)
ENDC
RETURN ( .T. )

FUNCTION HACharRel
* Sets the character width and height relative to P1 and P2 for labels.
PARA PlotPort,Width,Height
DO CASE
CASE PCOUNT()<1
  HACmnd(1,'SR')
CASE PCOUNT()<2
  HACmnd(PlotPort,'SR')
OTHE
  HACmnd(PlotPort,'SR',Width,Height)
ENDC
RETURN ( .T. )

FUNCTION HASelStand
PARA PlotPort
IF PCOUNT()<1
  PlotPort=1
ENDIF
HACmnd(PlotPort,'SR')
RETURN ( .T. )

FUNCTION HAUserChar
PARA PlotPort,Words
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd (PlotPort,'LB',Words+chr(3))
RETURN ( .T. )

FUNCTION HAVelocity
PARA PlotPort,Speed
IF PCOUNT()<2
  RETURN ( .F. )
ENDIF
HACmnd(PlotPort,'VS',Speed)
RETURN ( .T. )

FUNCTION HAPosAbs
PARA PlotPort,X,Y
IF PCOUNT()<3
  RETURN (.F.)
ENDIF
HACmnd(PlotPort,'PA')
HAPenUp(PlotPort,X,Y)
RETURN (.T.)

FUNCTION HAPosRel
PARA PlotPort,X,Y
IF PCOUNT()<3
  RETURN (.F.)
ENDIF
HACmnd(PlotPort,'PR')
HAPenUp(PlotPort,X,Y)
RETURN (.T.)

FUNCTION HACircle
* Draws a circle of specified radius at specified pen position
PARA PlotPort,X,Y,radius,chord
IF PCOUNT()<4
  RETURN ( .F. )
ENDIF
HAPosAbs(PlotPort,X,Y)
IF PCOUNT()<5
  HACmnd (PlotPort,'CI',radius)
ELSE
  HACmnd (PlotPort,'CI',radius,chord)
ENDIF
RETURN ( .T. )

FUNCTION HAFillWed
* Fills a wedge of specified radius
PARA PlotPort,radius,startA,sweepA,chord
IF PCOUNT()<4
  RETURN ( .F. )
ENDIF
IF PCOUNT()<5
  HACmnd (PlotPort,'WG',radius,startA,sweepA)
ELSE
  HACmnd (PlotPort,'WG',radius,startA,sweepA,chord)
ENDIF
RETURN ( .T. )

FUNCTION HAXTick
* Units defined by Tick Length
PARA PlotPort
IF PCOUNT()<1
  PlotPort=1
ENDIF
HACmnd (PlotPort,'XT')
RETURN ( .T. )

FUNCTION HAYTick
* Units defined by Tick Length
PARA PlotPort
IF PCOUNT()<1
  PlotPort=1
ENDIF
HACmnd (PlotPort,'YT')
RETURN ( .T. )

FUNCTION HAWindow
* Pass coordinates of window in inches, scaling units in integer values
* Will define a plotting window of the specified area with the scaling units
* passed to the function.
* According the the HP7475A "Interfacing and Programming Manual", p. 2-5,
* the plotter units are approximately 40.2/mm or 1021/inch.
*
PARA PlotPort,X1,Y1,X2,Y2,ScaleX1,ScaleX2,ScaleY1,ScaleY2
PRIV CalcX1,CalcY1,CalcX2,CalcY2,Units,Last
Last=PCOUNT()
IF Last<1
  PlotPort=1
ENDIF
IF Last<4
  HACmnd(PlotPort,'IW;IP')
  RETURN ( .T. )
ENDIF
IF X1>17.OR.X2>17.OR.Y1>17.OR.Y2>17 && units=mm
  Units=40.2
ELSE
  Units=1021
ENDIF
CalcX1=INT(X1*Units)
CalcX2=INT(X2*Units)
CalcY1=INT(Y1*Units)
CalcY2=INT(Y2*Units)
HACmnd(PlotPort,'IW;IP',CalcX1,CalcY1,CalcX2,CalcY2)
IF Last>6
  HACmnd(PlotPort,'SC',ScaleX1,ScaleX2,ScaleY1,ScaleY2)
ENDIF
RETURN ( .T. )

Function HAAxes
PARA PlotPort,XMin,XMax,YMin,YMax,XLab,YLab,XTick,YTick
PRIV I,ScaleX1,ScaleX2,ScaleY1,ScaleY2,XPoints,YPoints
I=PCOUNT()
IF I<5
  RETURN ( .F. )
ENDIF
DO CASE
CASE TYPE('YLab')='U'
  YLab=10
CASE TYPE('YLab')='N'
  YPoints=YLab
ENDC
DO CASE
CASE TYPE('XLab')='U'
  XLab=10
CASE TYPE('XLab')='N'
  XPoints=XLab
ENDC
IF TYPE('XPoints')='U'
  IF TYPE('Xlab')='A'
    XPoints=LEN(Xlab)
  ELSE
    XPoints=10
  ENDIF
ENDIF
IF TYPE('YPoints')='U'
  IF TYPE('Ylab')='A'
    YPoints=LEN(Ylab)
  ELSE
    YPoints=10
  ENDIF
ENDIF
XInc=ABS(XMin-XMax)/XPoints
YInc=ABS(YMin-YMax)/YPoints
IF I<8
  Tick=.T.
  XTick=0.5
ENDIF
IF I<9
  YTick=XTick
ENDIF
HATickLen(PlotPort,XTick,YTick)
HALineType(PlotPort)
ScaleX1=XMin-ABS(Xmin-XMax)/10
ScaleX2=XMax+ABS(Xmin-XMax)/10
ScaleY1=YMin-ABS(Ymin-YMax)/10
ScaleY2=YMax+ABS(Ymin-YMax)/10
HAScaling(PlotPort,ScaleX1,ScaleX2,ScaleY1,ScaleY2)
HAPlotAbs(PlotPort,ScaleX2,YMin,Xmin,Ymin)
HAPlotAbs(PlotPort,XMin,YMin,Xmin,ScaleY2)
I=XMin
DO WHIL I<=XMax
  HAPosAbs(PlotPort,I,Ymin)
  HAXTick(PlotPort)
  I=I+Xinc
ENDDO
I=YMin
DO WHIL I<=YMax
  HAPosAbs(PlotPort,Xmin,I)
  HAYTick(PlotPort)
  I=I+Yinc
ENDDO
RETURN ( .T. )

FUNCTION STRING
* Returns a character string of any data type
PARA S
IF PCOUNT()<1
  RETURN ( "" )
ENDIF
DO CASE
CASE TYPE('S')='U'
  RETURN ( "" )
CASE TYPE('S')='L'
  RETURN ( IF(S,'Yes','No ') )
CASE TYPE('S')='D'
  RETURN ( SUBSTR(CMONTH(S),1,3) )
CASE TYPE('S')='N'
  RETURN ( LTRIM(STR(S)) )
ENDC
RETURN ( S )

FUNCTION HAAxesLab
PARA PlotPort,XMin,XMax,YMin,YMax,XLab,YLab
PRIV I,J,ScaleX1,ScaleX2,ScaleY1,ScaleY2,XPoints,YPoints
i=PCOUNT()
IF I<5
  RETURN ( .F. )
ENDIF
DO CASE
CASE TYPE('YLab')='U'
  YLab=10
CASE TYPE('YLab')='N'
  YPoints=YLab
ENDC
DO CASE
CASE TYPE('XLab')='U'
  XLab=10
CASE TYPE('XLab')='N'
  XPoints=XLab
ENDC
IF TYPE('XPoints')='U'
  IF TYPE('Xlab')='A'
    XPoints=LEN(Xlab)
  ELSE
    XPoints=10
  ENDIF
ENDIF
IF TYPE('YPoints')='U'
  IF TYPE('Ylab')='A'
    YPoints=LEN(Ylab)
  ELSE
    YPoints=10
  ENDIF
ENDIF
XInc=ABS(XMin-XMax)/XPoints
YInc=ABS(YMin-YMax)/YPoints
ScaleX1=XMin-ABS(Xmin-XMax)/10
ScaleX2=XMax+ABS(Xmin-XMax)/10
ScaleY1=YMin-ABS(Ymin-YMax)/10
ScaleY2=YMax+ABS(Ymin-YMax)/10
HAScaling(PlotPort,ScaleX1,ScaleX2,ScaleY1,ScaleY2)
IF TYPE('XLab')='A'
  J=1
  LabVal='STRING(XLab[J])'
ELSE
  LabVal='Ntrim(I)'
ENDIF
I=XMin
DO WHIL I<=XMax
  Lab=&LabVal
  HAPosAbs(PlotPort,I-len(Lab)/2,YMin)
  HAMovByChr(PlotPort,0,-0.95)
  HALabel(PlotPort,Lab)
  I=I+Xinc
  IF TYPE('XLAB')='A'
    J=J+1
    IF J>Len(XLab)
      I=XMax+1
    ENDIF
  ENDIF
ENDDO
IF TYPE('Ylab')='A'
  J=1
  LabVal='STRING(YLab[J])'
ELSE
  LabVal='Ntrim(I)'
ENDIF
I=YMin
DO WHIL I<=YMax
  Lab=&LabVal
  HAPosAbs(PlotPort,XMin-Len(Lab),I)
  HAMovByChr(PlotPort,-1.25,0)
  HALabel(PlotPort,Lab)
  I=I+Yinc
  IF TYPE('YLAB')='A'
    J=J+1
    IF J>Len(YLab)
      I=YMax+1
    ENDIF
  ENDIF
ENDDO
RETURN ( .T. )

Function HATitle
* Code  Placement
* ====  ======================
*   1   Left top X axis
*   2   Mid top X axis
*   3   Right top X axis
*   4   Left bottom X axis
*   5   Mid bottom X axis
*   6   Right bottom X axis
*  -1   Left top Y axis
*  -2   Mid top Y axis
*  -3   Right top Y axis
*  -4   Left bottom Y axis
*  -5   Mid bottom Y axis
*  -6   Right bottom Y axis
PARA PlotPort,Title,Code,XMin,XMax,YMin,YMax
PRIV I,ScaleX1,ScaleX2,ScaleY1,ScaleY2,CurX,CurY,CurX
I=PCOUNT()
IF I<2
  RETURN ( .F. )
ENDIF
IF TYPE('Title')#'C'
  RETURN ( .F. )
ENDIF
IF I<3
  Code=1
ENDIF
IF I<5
  XMin=0
  XMax=100
ENDIF
IF I<7
  Ymin=Xmin
  YMax=Xmax
ENDIF
ScaleX1=XMin-ABS(Xmin-XMax)/10
ScaleX2=XMax+ABS(Xmin-XMax)/10
ScaleY1=YMin-ABS(Ymin-YMax)/10
ScaleY2=YMax+ABS(Ymin-YMax)/10
HAScaling(PlotPort,ScaleX1,ScaleX2,ScaleY1,ScaleY2)
HALabelAbs(PlotPort,1,0)
DO CASE
CASE Code>0
  IF Code<4
    YVal=ScaleY2
  ELSE
    YVal=ScaleY1
  ENDIF
  DO CASE
  CASE Code=1.OR.Code=4
    XVal=XMin
  CASE Code=2.OR.Code=5
    XVal=ABS(Xmin-XMax)/2-Len(Title)/2
  OTHE
    XVal=XMax-Len(Title)
  ENDC
CASE Code<0
  HALabelAbs(PlotPort,0,1)
  Code=Abs(Code)
  IF Code<4
    XVal=ScaleX2
  ELSE
    XVal=ScaleX1
  ENDIF
  DO CASE
  CASE Code=1.OR.Code=4
    YVal=YMin
  CASE Code=2.OR.Code=5
    YVal=ABS(Ymin-YMax)/2-Len(Title)/2
  OTHE
    YVal=YMax-Len(Title)
  ENDC
  Code=-Code
OTHE
  XVal=ABS(Xmin-XMax)/2-Len(Title)/2
  YVal=ABS(Ymin-YMax)/2
ENDC
HAPosAbs(PlotPort,XVal,YVal)
IF Code<0.AND.ABS(Code)<4
  HAMovByChr(PlotPort,0,-0.95)
ENDIF
HALabel(PlotPort,Title)
RETURN ( .F. )

Function HASetGraph
PARA PlotPort,XMin,XMax,YMin,YMax,Title,XLab,YLab
PRIV ScaleX1,ScaleX2,ScaleY1,ScaleY2,I,XPoints,YPoints
I=Pcount()
IF I<1
  PlotPort=1
ENDIF
IF I=2
  IF TYPE('XMin')='C'
    Title=XMin
  ENDIF
ENDIF
IF I<3
  Xmin=0
  Xmax=100
ENDIF
IF I<5
  Ymin=XMin
  Ymax=XMax
ENDIF
DO CASE
CASE TYPE('XLab')='U'
  XLab=10
CASE TYPE('XLab')='N'
  XPoints=XLab
ENDC
DO CASE
CASE TYPE('YLab')='U'
  YLab=10
CASE TYPE('YLab')='N'
  YPoints=YLab
ENDC
IF TYPE('XPoints')='U'
  IF TYPE('Xlab')='A'
    XPoints=LEN(Xlab)
  ELSE
    XPoints=10
  ENDIF
ENDIF
IF TYPE('YPoints')='U'
  IF TYPE('Ylab')='A'
    YPoints=LEN(Ylab)
  ELSE
    YPoints=10
  ENDIF
ENDIF
HASelPen(PlotPort,1)
HAPenUp(PlotPort)
IF TYPE('Title')#'U'
  IF TYPE('Title')='C'
    HATitle(PlotPort,Title,2,XMin,XMax,YMin,YMax)
  ENDIF
ENDIF
HASelPen(PlotPort,6)
HAAxes(PlotPort,XMin,XMax,Ymin,YMax,XPoints,YPoints)
* Placing labels if requested
IF TYPE('Xlab')='U'
  XLab=0
ENDIF
IF TYPE('Ylab')='U'
  YLab=0
ENDIF
HASelPen(PlotPort,4)
HAAxesLab(PlotPort,XMin,XMax,YMin,YMax,XLab,YLab)
HASelPen(PlotPort,0)
RETURN ( .T. )

FUNCTION HAPlotData
PARA PlotPort,XField,YField,Start,Stop,DFile,IFile
PRIV I,K,StopCond,RetArea,TX,TY
I=PCOUNT()
IF ! EMPTY(ALIAS())
  RETAREA=ALIAS()
ELSE
  RETAREA=''
ENDIF
IF I<3
  RETURN ( .F. )
ENDIF
IF EMPTY(ALIAS()).AND.I<5
  RETURN ( .F. )
ENDIF
IF EMPTY(ALIAS()) && A file to be selected
  RETAREA=ALIAS()
  SELE &DFILE
  IF EMPTY(ALIAS())
    SELE 0
    IF FILE(DFile+IF('.' $ DFILE,'','.DBF'))
      SELE &DFILE
    ELSE
      RETURN ( .F. )
    ENDIF
  ENDIF
ENDIF
IF i>6 && An index file to be selected
  IF FILE(DFile+IF('.' $ IFILE,'','.NTX'))
    SET INDE TO &DFILE
  ENDIF
ENDIF
DO CASE
CASE TYPE('XField')#'C'
  RETURN ( .F. )
CASE TYPE('YField')#'C'
  RETURN ( .F. )
ENDC
DO CASE
CASE ! TYPE(XField) $ 'ND'
  RETURN ( .F. )
CASE ! TYPE(YField) $ 'ND'
  RETURN ( .F. )
ENDC
DO CASE
CASE I<4
  GO TOP
  I=1
CASE Empty('Start')
  GO TOP
  I=1
CASE TYPE('Start')='N'
  GO START
  I=START
CASE TYPE('Start')='C'
  IF ! EMPTY(IndexKey(0))
    IK=IndexKey(0)
    IK=&IK
    IF TYPE('IK')=TYPE('Start')
      SEEK &START
    ENDIF
  ELSE
    LOCA FOR &Start
  ENDIF
ENDC
DO CASE
CASE I<4
  StopCond='.T.'
CASE TYPE('Stop')='N'
  StopCond='KOUNT<'+NTRIM(Stop)
CASE TYPE('Stop')='C'
  StopCond=Stop
ENDC
Kount=1
TX=&XField
TY=&YField
SKIP
HAPlotAbs(PlotPort,TX,TY,&XField,&YField)
DO WHIL &StopCond .AND. ! EOF()
  SKIP
  IF ! EOF()
    HAPlotAbs(PlotPort,&XField,&YField)
  ENDIF
  Kount=Kount+1
ENDDO
IF ! EMPTY(RetArea)
  SELE &RetArea
ENDIF
RETURN ( .T. )

FUNCTION HAPlotArray
PARA PlotPort,X,Y,Start,Stop
PRIV I,K
I=PCOUNT()
IF I<3
  RETURN ( .F. )
ENDIF
DO CASE
CASE TYPE('X')#'A'
  RETURN ( .F. )
CASE TYPE('Y')#'A'
  RETURN ( .F. )
ENDC
DO CASE
CASE I<4
  Start=1
CASE TYPE('Start')#'N'
  Start=1
ENDC
DO CASE
CASE I<5
  Stop=MIN(Len(X),Len(Y))
CASE TYPE('Start')#'N'
  Stop=MIN(Len(X),Len(Y))
ENDC
IF Start=Stop
  HAPlotAbs(X[Start],Y[Start])
  RETURN ( .T. )
ENDIF
Kount=Start+1
HAPlotAbs(PlotPort,X[Start],Y[Start],X[Kount],Y[Kount])
DO WHIL Kount-1<Stop.AND.Kount-1<MIN(Len(X),Len(Y))
  Kount=Kount+1
  HAPlotAbs(PlotPort,X[Kount],Y[Kount])
ENDDO
RETURN ( .T. )
